﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Interactivity;

namespace ReflectionBehavior
{
    /// <summary>
    /// Reflection Behavior <c>Silverlight</c> 3
    /// </summary>
    public class ReflectionBehavior : Behavior<Canvas>
    {
        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();
            AssociatedObject.SizeChanged += AssociatedObject_SizeChanged;
            createReflect();
        }

        void AssociatedObject_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            createReflect();
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            suppressMarker(AssociatedObject);
        }

        /// <summary>
        /// Gets or sets the Y offset. 0 means the reflected image begins exactly at bottom side of
        /// original. A little positive offset is generally nicer.
        /// </summary>
        public static readonly DependencyProperty YOffsetProperty =
            DependencyProperty.Register("YOffset",
            typeof(int), typeof(ReflectionBehavior), new PropertyMetadata(3));

        /// <summary>
        /// Gets or sets the Y offset. 0 means the reflected image begins exactly at bottom side of
        /// original. A little positive offset is generally nicer.
        /// </summary>
        /// <value>The Y offset.</value>
        public int YOffset
        {
            get { return (int)GetValue(YOffsetProperty); }
            set { SetValue(YOffsetProperty, value); }
        }

        /// <summary>
        /// Gets or sets the opacity "from".
        /// </summary>
        public static readonly DependencyProperty OpacityFromProperty =
            DependencyProperty.Register("OpacityFrom",
            typeof(double), typeof(ReflectionBehavior), new PropertyMetadata(0.5));


        /// <summary>
        /// Gets or sets the opacity "from".
        /// </summary>
        /// <value>The opacity from.</value>
        public double OpacityFrom
        {
            get { return (double)GetValue(OpacityFromProperty); }
            set { SetValue(OpacityFromProperty, value); }
        }


        /// <summary>
        /// Gets or sets the opacity to.
        /// </summary>
        public static readonly DependencyProperty OpacityToProperty =
           DependencyProperty.Register("OpacityTo", 
           typeof(double), typeof(ReflectionBehavior), new PropertyMetadata(0.0));

        /// <summary>
        /// Gets or sets the opacity to.
        /// </summary>
        /// <value>The opacity to.</value>
        public double OpacityTo
        {
            get { return (double)GetValue(OpacityToProperty); }
            set { SetValue(OpacityToProperty, value); }
        }

        private void suppressMarker(Canvas source)
        {
            if (source==null) return;
            for (var i=source.Children.Count-1;i>=0;i--)
            {
                if ((source.Children[i] is Image) && (((Image)source.Children[i]).Tag==marker))
                    source.Children.RemoveAt(i);
            }
        }

        private const string marker = "ReflectionBehavior_Generated_Reflect";
        private void createReflect()
        {
            suppressMarker(AssociatedObject);
            var theReflect = new Image();
            theReflect.Tag = marker;
            var bitmap =
               new WriteableBitmap((int)AssociatedObject.Width, (int)AssociatedObject.Height);
            bitmap.Render(AssociatedObject, null);
            bitmap.Invalidate();
            theReflect.Source = bitmap;
            var tFlipY = new ScaleTransform {ScaleX = 1.0, ScaleY = (-1.0)};
            var tTranslateY = new TranslateTransform {Y = (AssociatedObject.Height*2) + YOffset};
            var tGroup = new TransformGroup();
            tGroup.Children.Add(tFlipY);
            tGroup.Children.Add(tTranslateY);
            theReflect.RenderTransform = tGroup;
            var brush = new LinearGradientBrush();
            var gStop1 = new GradientStop
                            {
                                Color = Color.FromArgb((byte) (OpacityTo*255), 0, 0, 0),
                                Offset = 0.0
                            };
            var gStop2 = new GradientStop
                            {
                                Color = Color.FromArgb((byte) (OpacityFrom*255), 0, 0, 0),
                                Offset = 1.0
                            };
            brush.StartPoint = new Point(0.5, 0);
            brush.EndPoint = new Point(0.5, 1);
            brush.GradientStops.Add(gStop1);
            brush.GradientStops.Add(gStop2);
            theReflect.OpacityMask = brush;
            AssociatedObject.Children.Add(theReflect);
        }
    }
}